//-----------------------------------------------------------------
// CSE 461 Project
//
// debug.java: an interface for selective debug messages
//-----------------------------------------------------------------
package debug;

//-----------------------------------------------------------------
// Debug: the debugging output interface.  This class is like
//        System in that it is static.  Use print and println
//        for messages, Enable and Disable turn specific
//        categories on and off.
//-----------------------------------------------------------------
/**
  Debug message system.  This class supports printing of selectable
  debugging messages.  All debugging messages are associated with
  a character identifier.  This character is passed to the printing
  methods, and the set of characters which are enabled can be adjusted 
  dynamically.
*/
public class Debug
{
   private Debug ()
   {
      // Debug objects are never instantiated
   }

   private static String enabled = "";

/**
   Enable the specified character.  Does not affect other enabled characters 

@param type the debug message class to enable
*/
   public static void Enable (char type)
   {
      if (!IsEnabled (type))
         enabled = enabled + type;
   }

/**
   Disable the specified character.  Does not affect other enabled characters 

@param type the debug message class to disable
*/
   public static void Disable (char type)
   {
      int i = enabled.indexOf (type);
      if (i >= 0)
         enabled = enabled.substring (0, i) + enabled.substring (i+1);
   }

/**
   Set the set of enable characters to precisely those specified.
   
@param types the string of debug message classes to enable
*/
   public static void SetEnabled (String types)
   {
      enabled = types;
   }

/**
   Query if a particular character is enabled
@param type the class of debug message to check
@returns <tt>true</tt> if and only if the messages for that class 
  are enabled
*/
   public static boolean IsEnabled (char type)
   {
      return (enabled.indexOf (type) >= 0);
   }

/**
   Print a debug message

@param type the class for this message
@param s the message to print
*/
   public static void print (char type, String s)
   {
      if (IsEnabled (type))
         System.out.print (s);
   }

/**
   Print a debug message followed by a newline

@param type the class for this message
@param s the message to print
*/
   public static void println (char type, String s)
   {
      if (IsEnabled (type))
         System.out.println (s);
   }
}
