package link;

/**
   Interface for a generic node in a network.  Nodes connect to 
   links, and transmit and receive packets on those links to
   implement network protocols.  Actual node implementations
   will generally be found in the network layer, but the interface
   is here because of the relationship with links.
*/
public interface Node
{
/**
   Callback when a packet arrives at the node.  Called by
   the links to which this node is connected.

@param p the packet which arrived
@param l the link on which the packet arrived
@see Link
*/
   public void PacketArrived (LinkPacket p, Link l);

/**
   Connect this node to a link.  This is called for the convenience
   of the node.  Packets will arrive as a result of this node being
   registered with the link, independant of whether the link is
   registered with the node.  Clients should, however, avoid such 
   "half-connections."

@param l the link being connected
*/
   public void AddLink (Link l);

/** 
   Disconnect this node from a link.  The inverse of <tt>AddLink</tt>.

@param l the link being disconnected
*/
   public void RemoveLink (Link l);
}

