package network;

/**
   Interface for a generic network.  Networks support sending
   and receiving of addressed packets.  Clients using a network
   will need to implement the NetworkClient interface to 
   receive packets.  Packets are submitted to the network
   with the <tt>TransmitPacket</tt> function.
*/
public interface Network
{
/**
   Submit a packet to the network, for delivery to the
   client whose address is in the <tt>destinationAddr</tt> field
   of the packet.  Blocks for whatever transmission time the
   network imposes.

   @param p the packet to transmit.
*/
   public void TransmitPacket (NetworkPacket p);

/**
   Query the maximum transmission size of the network.

   @return the maximum transmission unit, in bytes
*/
   public int MTU ();

/**
   Connect an addressed client to the network.  Any packets 
   which arrive with this client's address will now be routed to 
   this client by calling the <tt>PacketArrived</tt> method.  

@param client the network client wishing to receive packets
@param address the address of this client
@see PacketReceiver
*/
   public void AddReceiver (PacketReceiver client, Address address);

/** 
   Disconnect this client from the network.

@param client the network client which will no longer receive
   packets
*/
   public void RemoveReceiver (PacketReceiver clinet);
}
