package network;

import link.*;

/***************************************************************** 
   The extra data carried by a network-level packet.  This 
   extends the <tt>LinkPacket</tt> structure in the <tt>link</tt>
   package, from which it inherits the field
<pre>   public byte[] data;
</pre>
   The network level adds addressing.  <tt>NetworkPacket</tt> 
   is extended by the reliability layer with sequence numbers
   and control bits.
   <p>
   Like all packets, NetworkPacket implements a toString
   for use by packet tracing.
   <p>
   While you are programming in the network level, feel free
   to add reasonable fields here.

@see LinkPacket
 *****************************************************************/
public class NetworkPacket extends LinkPacket
{
   /************************************************************
      Constructor for some default setup.  The defaults are
      <ul>
      <li> timeToLive = 10
      </ul>
    ************************************************************/
   public NetworkPacket ()
   {
      timeToLive = 10;
   }

   /************************************************************
      Make a string for packet trace messages.  Should be short
    ************************************************************/
   public String toString ()
   {
      return "(Pack:" + destinationAddr + ")";
   }

   /************************************************************
     Address of the packet's sender
    ************************************************************/
   public Address sourceAddr;

   /************************************************************
     Address of the packet's intended recipient
    ************************************************************/
   public Address destinationAddr;

   /************************************************************
      Maximum number of hops before this packet dies
    ************************************************************/
   public int timeToLive;
}
