package network;

import debug.*;
import timer.*;
import link.*;

/**
   Interface for a generic network.  Networks support sending
   and receiving of addressed packets.  Clients using a network
   will need to implement the NetworkClient interface to 
   receive packets.  Packets are submitted to the network
   with the <tt>TransmitPacket</tt> function.
*/
public class RoutingNetworkEdge implements Network, Node
{
   public RoutingNetworkEdge ()
   {
      receiver = null;
      address = null;
      link = null;
   }

   public String toString ()
   {
      return "Edge:<" + address + ">";
   }

   public void TransmitPacket (NetworkPacket p)
   {
       Debug.println ('e', "RoutedNetworkEdge: " + address +
                           ": message arrived for transmit at " + 
                           Timer.global.CurrentSimTime ());
       if (link != null)
         link.TransmitPacket (p, this);
   }

   public int MTU ()
   {
      return 20;
   }

   public void AddReceiver (PacketReceiver client, 
                            Address address)
   {
      if (receiver == null)
         {
         receiver = client;
         this.address = address;
         }
   }

   public void RemoveReceiver (PacketReceiver client)
   {
      receiver = null;
      address = null;
   }

   public void PacketArrived (LinkPacket p, Link l)
   {
      NetworkPacket packet = (NetworkPacket) p;
      if (packet.destinationAddr.equals (address))
         receiver.PacketArrived (packet);
      else
         Debug.println ('r', "" + Timer.global.CurrentSimTime () +
                             ": network edge at " + address +
                             " dropping packet for " + packet.destinationAddr);
   }

   /************************************************************
      Connect a link
      @see Node#AddLink
    ************************************************************/
   public void AddLink (Link l)
   {
      if (link == null)
         link = l;
   }

   /************************************************************
      Disconnect a link
      @see Node#RemoveLink
    ************************************************************/
   public void RemoveLink (Link l)
   {
      link = null;
   }

   private PacketReceiver receiver;
   private Address address;
   private Link link;
}

