package network;

import debug.*;
import timer.*;
import link.*;
import reliability.*;
import application.*;

import java.io.*;
import java.util.*;

public class project2
{
   public static void main (String[] args)
   {
      project2 runProject = new project2 ();
      ParseArgs (args);
      runProject.BuildNetworkConfig ();
   }

   public static long SEED = System.currentTimeMillis ();

   public static String CONFIG_FILENAME = "project2.cfg";

   public static long STARTUP_DELTA = 5000;
   public static long SHUTDOWN_DELTA = 5000;

   private static void ParseArgs (String[] args)
   {
      for (int i=0; i<args.length; ++i)
         {
         int eqidx = args[i].indexOf ('=');
         if (eqidx == -1)
            continue;

         String prefix = args[i].substring (0, eqidx);
         String value = args[i].substring (eqidx + 1);

         if (prefix.equals ("seed"))
            SEED = ParseLong (value, 0);

         else if (prefix.equals ("config"))
            {
            CONFIG_FILENAME = value;
            }

         else if (prefix.equals ("debug"))
            Debug.SetEnabled (value);
         }

      for (int i=0; i<args.length; ++i)
         {
         if (args[i].equals ("-flag"))
            {
            }
         }
   }

   private static long ParseLong (String s, long def)
   {
      try
         {
         def = Long.parseLong (s);
         }
      catch (NumberFormatException e)
         {
         }

      return def;
   }

   private static double ParseDouble (String s, double def)
   {
      try 
         {
         def = Double.valueOf (s).doubleValue ();
         }
      catch (NumberFormatException e)
         {
         }
      catch (NullPointerException e)
         {
         }

      return def;
   }

   private Hashtable routers;
   private Hashtable links;
   private Hashtable networks;
   private Hashtable edges;
   private Hashtable senders;
   private Hashtable receivers;
   private Hashtable sendApps;
   private Hashtable receiveApps;

   private void BuildNetworkConfig ()
   {
      routers = new Hashtable ();
      links = new Hashtable ();
      networks = new Hashtable ();
      edges = new Hashtable ();
      senders = new Hashtable ();
      receivers = new Hashtable ();
      sendApps = new Hashtable ();
      receiveApps = new Hashtable ();

      try
         {
         FileInputStream file = new FileInputStream (CONFIG_FILENAME);
         StreamTokenizer input = new StreamTokenizer (file);
         input.commentChar ('#');

         for (;;)
            {
            int ttype = input.nextToken ();
            if (ttype == StreamTokenizer.TT_EOF)
               return;

            if (ttype != StreamTokenizer.TT_WORD)
               throw new InputParseError (input.lineno ());

            if (input.sval.equals ("Router"))
               {
               // Expect name and queue len
               if (input.nextToken () != '\"')
                  throw new InputParseError (input.lineno ());
               String name = input.sval;

               if (input.nextToken () != StreamTokenizer.TT_NUMBER)
                  throw new InputParseError (input.lineno ());
               int queueLen = (int) input.nval;

               if (!routers.containsKey (name))
                  routers.put (name, new Router (name, queueLen));
               }
            else if (input.sval.equals ("Link"))
               {
               Link link = null;

               // Expect name, class, end1, end2, 
               //         mtu, drop prob, bandwidth mean/sd/walk
               // and sometimes latency mean/sd/walk

               if (input.nextToken () != '\"')
                  throw new InputParseError (input.lineno ());
               String name = input.sval;

               if (input.nextToken () != StreamTokenizer.TT_WORD)
                  throw new InputParseError (input.lineno ());
               String kind = input.sval;

               if (input.nextToken () != '\"')
                  throw new InputParseError (input.lineno ());
               String end1 = input.sval;
               if (!routers.containsKey (end1))
                  throw new InputParseError (input.lineno ());

               if (input.nextToken () != '\"')
                  throw new InputParseError (input.lineno ());
               String end2 = input.sval;
               if (!routers.containsKey (end2))
                  throw new InputParseError (input.lineno ());


               if (input.nextToken () != StreamTokenizer.TT_NUMBER)
                  throw new InputParseError (input.lineno ());
               int mtu = (int) input.nval;

               if (input.nextToken () != StreamTokenizer.TT_NUMBER)
                  throw new InputParseError (input.lineno ());
               double dropProb = input.nval;

               if (input.nextToken () != StreamTokenizer.TT_NUMBER)
                  throw new InputParseError (input.lineno ());
               double bandwidthMean = input.nval;

               if (input.nextToken () != StreamTokenizer.TT_NUMBER)
                  throw new InputParseError (input.lineno ());
               double bandwidthSD = input.nval;

               if (input.nextToken () != StreamTokenizer.TT_NUMBER)
                  throw new InputParseError (input.lineno ());
               double bandwidthWalk = input.nval;

               if (kind.equals ("SimpleBroadcastLink"))
                  {
                  Vector downPeriods = null;

                  if (input.nextToken () != StreamTokenizer.TT_NUMBER)
                     throw new InputParseError (input.lineno ());
                  int start = (int) input.nval;

                  if (start != -1)
                     {
                     downPeriods = new Vector ();
                     while (start != -1)
                        {
                        if (input.nextToken () != StreamTokenizer.TT_NUMBER)
                           throw new InputParseError (input.lineno ());
                        int end = (int) input.nval;

                        TimeInterval t = new TimeInterval ();
                        t.start = start;
                        t.end = end;
                        downPeriods.addElement (t);

                        if (input.nextToken () != StreamTokenizer.TT_NUMBER)
                           throw new InputParseError (input.lineno ());
                        start = (int) input.nval;
                        }
                    }

                  link =  new SimpleBroadcastLink (SEED, mtu, 
                                      bandwidthMean, bandwidthSD, 
                                      bandwidthWalk, dropProb, downPeriods);
                  }
               else
                  {
                  // expect latency mean/sd/walk
                  if (input.nextToken () != StreamTokenizer.TT_NUMBER)
                     throw new InputParseError (input.lineno ());
                  double latencyMean = input.nval;

                  if (input.nextToken () != StreamTokenizer.TT_NUMBER)
                     throw new InputParseError (input.lineno ());
                  double latencySD = input.nval;

                  if (input.nextToken () != StreamTokenizer.TT_NUMBER)
                     throw new InputParseError (input.lineno ());
                  double latencyWalk = input.nval;

                  Vector downPeriods = null;

                  if (input.nextToken () != StreamTokenizer.TT_NUMBER)
                     throw new InputParseError (input.lineno ());
                  int start = (int) input.nval;

                  if (start != -1)
                     {
                     downPeriods = new Vector ();
                     while (start != -1)
                        {
                        if (input.nextToken () != StreamTokenizer.TT_NUMBER)
                           throw new InputParseError (input.lineno ());
                        int end = (int) input.nval;

                        TimeInterval t = new TimeInterval ();
                        t.start = start;
                        t.end = end;
                        downPeriods.addElement (t);

                        if (input.nextToken () != StreamTokenizer.TT_NUMBER)
                           throw new InputParseError (input.lineno ());
                        start = (int) input.nval;
                        }
                    }

                  if (kind.equals ("SimplePointToPointLink"))
                     {
                     link = new SimplePointToPointLink (name, SEED, 
                                         mtu, 
                                         bandwidthMean, bandwidthSD, 
                                         bandwidthWalk, 
                                         latencyMean, latencySD, 
                                         latencyWalk, 
                                         dropProb, downPeriods);
                     }
                  }

               if (link != null)
                  {
                  if (!links.containsKey (name))
                     links.put (name, link);
                  Node endNode1 = (Node) routers.get (end1);
                  Node endNode2 = (Node) routers.get (end2);

                  endNode1.AddLink (link);
                  link.AddNode (endNode1);

                  endNode2.AddLink (link);
                  link.AddNode (endNode2);
                  }
               else
                  throw new InputParseError (input.lineno ());

               }
            else if (input.sval.equals ("Sender"))
               {
               // Expect name, build sender and edge

               if (input.nextToken () != '\"')
                  throw new InputParseError (input.lineno ());
               String name = input.sval;

               Address address = new Address (name);

               RoutingNetworkEdge edge = new RoutingNetworkEdge ();
               ReliableSender sender = new SlidingWindowSender (edge, address);

               if (!routers.containsKey (name))
                  routers.put (name, edge);
               if (!senders.containsKey (name))
                  senders.put (name, sender);
               }
            else if (input.sval.equals ("Receiver"))
               {
               // Expect name, build sender and edge
               if (input.nextToken () != '\"')
                  throw new InputParseError (input.lineno ());
               String name = input.sval;

               Address address = new Address (name);

               RoutingNetworkEdge edge = new RoutingNetworkEdge ();
               ReliableReceiver receiver = new SlidingWindowReceiver (edge, address);

               if (!routers.containsKey (name))
                  routers.put (name, edge);
               if (!receivers.containsKey (name))
                  receivers.put (name, receiver);
               }

            else if (input.sval.equals ("Load"))
               {
               System.out.println ("Got load");
               // expect sender, receiver, list of intervals
               if (input.nextToken () != '\"')
                  throw new InputParseError (input.lineno ());
               String sendName = input.sval;

               if (!senders.containsKey (sendName))
                  throw new InputParseError (input.lineno ());

               if (input.nextToken () != '\"')
                  throw new InputParseError (input.lineno ());
               String receiveName = input.sval;
               
               if (!receivers.containsKey (receiveName))
                  throw new InputParseError (input.lineno ());

               Vector upPeriods = null;

               if (input.nextToken () != StreamTokenizer.TT_NUMBER)
                  throw new InputParseError (input.lineno ());
               int start = (int) input.nval;

               if (start != -1)
                  {
                  upPeriods = new Vector ();
                  while (start != -1)
                     {
                     if (input.nextToken () != StreamTokenizer.TT_NUMBER)
                        throw new InputParseError (input.lineno ());
                     int end = (int) input.nval;

                     TimeInterval t = new TimeInterval ();
                     t.start = start;
                     t.end = end;
                     upPeriods.addElement (t);

                     if (input.nextToken () != StreamTokenizer.TT_NUMBER)
                        throw new InputParseError (input.lineno ());
                     start = (int) input.nval;
                     }
                  }

               BytestreamSenderClient sender = 
                           new BytestreamSenderClient ((ReliableSender) senders.get (sendName),
                                                       upPeriods, 
                                                       new Address (sendName),
                                                       new Address (receiveName));

               if (upPeriods != null)
                  {
                  Vector up2 = new Vector ();
                  for (int i = 0; i < upPeriods.size (); ++i)
                     {
                     TimeInterval t = (TimeInterval) upPeriods.elementAt (i);
                     TimeInterval t2 = new TimeInterval ();
                     t2.start = t.start + STARTUP_DELTA;
                     t2.end   = t.end + SHUTDOWN_DELTA;
                     up2.addElement (t2);
                     }
                  upPeriods = up2;
                  }

               BytestreamReceiverClient receiver = 
                           new BytestreamReceiverClient ((ReliableReceiver) receivers.get (receiveName),
                                                         upPeriods, 
                                                         new Address (receiveName));

               sendApps.put (sendName, sender);
               receiveApps.put (receiveName, receiver);
               }
            }
         }
      catch (IOException e)
         {
         }
   }
}

class InputParseError extends RuntimeException
{
   public InputParseError (int lineno)
   {
      super ("Parse error at input line: " + lineno);
   }
}
