package reliability;

import network.*;

/***************************************************************** 
   The extra data carried by a reliability-level packet.  This 
   extends the <tt>NetworkPacket</tt> structure in the <tt>network</tt>
   package
   <p>
   While you are programming in the reliability level, feel free
   to add reasonable fields here.

@see NetworkPacket
 *****************************************************************/
public class ReliablePacket extends NetworkPacket
{
   /************************************************************
      Constructor for some default setup.  The defaults are
      <ul>
      <li> ack = false;
      <li> syn = false;
      <li> fin = false;
      </ul>
    ************************************************************/
   public ReliablePacket ()
   {
      ack = false;
      syn = false;
      fin = false;
   }

   /************************************************************
      Make a string for packet trace messages.  Should be short
    ************************************************************/
   public String toString ()
   {
      String id = "";

      if (syn)
         id = id + "S";
      if (fin)
         id = id + "F";
      if (ack)
         id = id + "A";

      if (!syn && !fin && !ack)
         id = "D";

      return "(" + id + sequenceNum + 
             ":" + destinationAddr + ")";
   }

   /************************************************************
     Is this packet an ack?
    ************************************************************/
   public boolean ack; 

   /************************************************************
     Sequence number for this packet.  
    ************************************************************/
   public int sequenceNum; 

   /************************************************************
     Is this packet an syn?
    ************************************************************/
   public boolean syn; 

   /************************************************************
     Is this packet a fin?
    ************************************************************/
   public boolean fin; 
}
