package reliability;
import debug.*;
import timer.*;
import network.*;
import link.*;

import java.io.*;
import java.util.*;

/***************************************************************** 
   The interface for a reliable bytestream receiver implementation.
   This version supports connection setup and teardown.  The 
   sender and receiver are separated, and each is permitted only
   one open connection at a time.

   @see ReliableSender
 *****************************************************************/
public interface ReliableReceiver
{
   /************************************************************
      Receive a block of data from the connection.  Data blocks 
      of any size may be received, and the operation will block
      until that much data is available.  The only exception is
      that if the connection is closed or becomes closed before
      the requested amount of data is received, then less data
      may be returned.  

      @param size the number of bytes requested
      @return the data received, up to the number of bytes
              requested
    ************************************************************/
   public byte[] Receive (int size);
}

