package reliability;
import debug.*;
import timer.*;
import network.*;
import link.*;

import java.io.*;
import java.util.*;

/***************************************************************** 
   The interface for a reliable bytestream sender implementation.
   This version supports connection setup and teardown.  The 
   sender and receiver are separated, and each is permitted only
   one open connection at a time.

   @see ReliableReceiver
 *****************************************************************/
public interface ReliableSender
{
   /************************************************************
      Open a connection to the ReliableReceiver represented by
      the address.  This call blocks until the connection is
      established.
 
      @param destination the address of the receiver
    ************************************************************/
   public void Open (Address destination);

   /************************************************************
      Send a block of data on the connection.  Any amount of
      data may be sent.  This call may block if there is 
      insufficient buffer space.  Clients may make any number
      of <tt>Send</tt> calls per open connection.

      @param data the data (any amount) to send on the connection
      @throws ConnectionNotOpenException if the connection is
          not open
    ************************************************************/
   public void Send (byte[] data) throws ConnectionNotOpenException;

   /************************************************************
      Close the current connection.  Blocks until all data
      is read by the other application and the close
      operation completes.

      @throws ConnectionNotOpenException if the connection is
          not open
    ************************************************************/
   public void Close () throws ConnectionNotOpenException;
}

