//-----------------------------------------------------------------
// CSE 461 Project 1:                   Provided client code
//
// This file implements the main driver loop for the program.
// It creates a sender and a receiver, connects them, and then
// sends a stream of data and measures the performance.  You
// shouldn't need to change anything but parameters in this file.
//
// See sender.java and receiver.java for the implementations
// of the sender and receiver algorithms
//
// USAGE:  java project1 seed=<num>
//-----------------------------------------------------------------
package reliability;

import debug.*;
import timer.*;
import network.*;

import java.io.*;
import java.util.*;

public class project1 implements Runnable
{
   // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
   // main entry point.  Create an actual instance and call that.
   // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
   public static void main (String[] args)
   {
      project1 runProject = new project1 ();
      ParseArgs (args);
      runProject.Go (args);
   }

   public static long SEED = System.currentTimeMillis ();

   public static int LINK_MTU = 20;

   public static double LINK_BW = (0.5+0.5);
   public static double LINK_BW_SD = 0.2;
   public static double LINK_BW_WALK = 0.5;

   public static double LINK_LATENCY = 2.0;
   public static double LINK_LATENCY_SD = 0.5;
   public static double LINK_LATENCY_WALK = 0.5;

   public static double LINK_DROP_PROB = 0.25;

   private static void ParseArgs (String[] args)
   {
      for (int i=0; i<args.length; ++i)
         {
         int eqidx = args[i].indexOf ('=');
         if (eqidx == -1)
            continue;

         String prefix = args[i].substring (0, eqidx);
         String value = args[i].substring (eqidx + 1);

         if (prefix.equals ("seed"))
            SEED = ParseLong (value, 0);

         else if (prefix.equals ("bandwidth"))
            {
            LINK_BW = ParseDouble (value, (0.5+0.5));
            LINK_BW_SD = LINK_BW / 5.0;
            }
         else if (prefix.equals ("bandwidth-walk"))
            LINK_BW_WALK = ParseDouble (value, (0.5+0.5));

         else if (prefix.equals ("latency"))
            {
            LINK_LATENCY = ParseDouble (value, (0.5+0.5));
            LINK_LATENCY_SD = LINK_LATENCY / 4.0;
            }
         else if (prefix.equals ("latency-walk"))
            LINK_LATENCY_WALK = ParseDouble (value, (0.5+0.5));

         else if (prefix.equals ("drop-prob"))
            LINK_DROP_PROB = ParseDouble (value, 0.25);

         else if (prefix.equals ("debug"))
            Debug.SetEnabled (value);
         }

      for (int i=0; i<args.length; ++i)
         {
         if (args[i].equals ("-no-bandwidth-variation"))
            LINK_BW_SD = 0.0;
         if (args[i].equals ("-no-latency-variation"))
            LINK_LATENCY_SD = 0.0;
         }
   }

   private static long ParseLong (String s, long def)
   {
      try
         {
         def = Long.parseLong (s);
         }
      catch (NumberFormatException e)
         {
         }

      return def;
   }

   private static double ParseDouble (String s, double def)
   {
      try 
         {
         def = Double.valueOf (s).doubleValue ();
         }
      catch (NumberFormatException e)
         {
         }
      catch (NullPointerException e)
         {
         }

      return def;
   }

   // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
   // The main run thread: set up everything, and then run the
   // sender, which blasts a stream of data across the link
   // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
//   private InternetLinkMonitor monitor;

   private void Go (String[] args)
   {
      Thread.currentThread ().setName ("Project 1 data sender");
      Timer.global.RegisterThread (Thread.currentThread ());
      synchronized (Timer.global)
         {
         Network link = new SimpleUnreliableNetwork (LINK_MTU,
                                LINK_BW, LINK_BW_SD, LINK_BW_WALK,
                                LINK_LATENCY, LINK_LATENCY_SD, LINK_LATENCY_WALK,
                                LINK_DROP_PROB, SEED);
         Address senderAddr = new Address ("sender");
         Address receiverAddr = new Address ("receiver");

         ReliableSender sender = new SlidingWindowSender (link, senderAddr);
         receiver = new SlidingWindowReceiver (link, receiverAddr);


         sender.Open (receiverAddr);

         Thread rcvThread = new Thread (this);
         rcvThread.start ();

         byte[] data = new byte[50];
         String s = "";
         int count = 0;
         for (;;)
            {
            while (s.length () < 50)
               {
               s = s + " " + count;
               ++count;
               }

            s.getBytes (0, 50, data, 0);
            s = s.substring (50);

            sender.Send (data);
            }
         }
   }

   private ReliableReceiver receiver;

   // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
   // recevier thread: receive and print data, and compute the
   // observed bandwidth of the link as the time to receive
   // bytes.
   // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
   public void run ()
   {
      Thread.currentThread ().setName ("Project 1 data recevier");
      Timer.global.RegisterThread (Thread.currentThread ());
      synchronized (Timer.global)
         {
         long oldTime = Timer.global.CurrentSimTime ();
         int linesReceived = 0;
         for (;;)
            {
            byte[] data = receiver.Receive (40);
            long newTime = Timer.global.CurrentSimTime ();

            ++linesReceived;

            String s = new String (data, 0);
            System.out.println ("RECEIVED: '" + s + "' ("
                    + (float)((8.0 * 40.0 * linesReceived)
                              / (newTime - oldTime)) + " Mb/sec)");

            if (linesReceived > 50)
               {
//               monitor.PrintStats ();
               System.exit (0);
               }
            }
         }
   }
}

